@@ -1,32 +1,33 @@
 #!/usr/bin/perl
+# Last Update by ./update_subroutines.pl: Thu Jan  2 03:25:32 GMT 1997
 #______________________________________________________________
 # Title     : fetch_seq.pl
-# Usage     :
+# Usage     : fetch.pl SWISS_ENTRY
 # Function  : fetches swissprot entry or fasta format seq with
 #             given seq name(like  SAA_HORSE, SA*HORSE, SAA,..)
 #             you can give multi files(SAA*, SAU*) at the same
 #             time. This uses ENV setting of 'SWDIR'.
 #             If you already know where you put seq.dat you can
-#             specify in the prompt.
+#             specify in the prompt.  jong@mrc-lmb.cam.ac.uk
 # Example   :  fetch_seq.pl YAPT
-# Warning   :
-# Class     : Bio
 # Keywords  : fetch_swissprot_sequence, fetch_sequence,
 #             find_swiss_sequence, find_sequence
 # Options   : _  for debugging.
 #             #  for debugging.
 #             -f for fasta format file output
-# Package   : Bio
-# Reference : http://sonja.acad.cai.cam.ac.uk/perl.html
-# Returns   :
-# Tips      :
+#             -a is for ALL matched seq. (same as using glob=> *YEAST)
+#             -c is for Creating seq.idx file
+#             -h is for HELP!
+#             -g is for GDF file format output
+#             -l is for list of match entries(in 1 column)
+#             -s is for species option (input name mst be species (YEAST, RAT, HUMAN..)
+#             n= is for Number of seq you want to get from swissprot
+#             s= is for Size limit. Min seq size in swiss, s=10  -> minimum 11 aa seq.
+#             S= is for Size limit. Max seq size in swiss, s=1000 -> get less than 1000
+#
+# Returns   : STDOUT
 # Argument  : swissprot seqname
-# Todo      :
-# Author    : Alex Bateman, Bissan, Sarah Teichmann, Astrid Reinhardt
-#             agb@mrc-lmb.cam.ac.uk
-# Version   : 1.0
-# Used in   :
-# Enclosed  :
+# Version   : 1.6
 #--------------------------------------------------------------
 
 
@@ -35,55 +36,37 @@
 
 #______________________________________________________________
 # Title     : fetch_seq
-# Usage     :
+# Usage     : &fetch_seq(@ARGV);
 # Function  : fetches swissprot entry or fasta format seq with
 #             given seq name(like  SAA_HORSE, SA*HORSE, SAA,..)
 #             you can give multi files(SAA*, SAU*) at the same
 #             time. This uses ENV setting of 'SWDIR'
 # Example   : &fetch_swiss_seq(@ARGV);
-# Warning   :
-# Class     : Bio
 # Keywords  : fetch_swissprot_sequence, fetch_sequence,
 #             find_swiss_sequence, find_sequence
 # Options   : _  for debugging.
 #             #  for debugging.
 #             -f for fasta format file output
-# Package   : Bio
-# Reference : http://sonja.acad.cai.cam.ac.uk/perl_for_bio.html
-# Returns   :
-# Tips      :
+#             -a is for ALL matched seq. (same as using glob=> *YEAST)
+#             -c is for Creating seq.idx file
+#             -h is for HELP!
+#             -g is for GDF file format output
+#             -l is for list of match entries(in 1 column)
+#             -s is for species option (input name mst be species (YEAST, RAT, HUMAN..)
+#             n= is for Number of seq you want to get from swissprot
+#             s= is for Size limit. Min seq size in swiss, s=10  -> minimum 11 aa seq.
+#             S= is for Size limit. Max seq size in swiss, s=1000 -> get less than 1000
+#
 # Argument  : swissprot seqname
-# Todo      :
-# Author    : A Scientist
-# Version   : 1.4
-# Used in   :
-# Enclosed  :
+# Version   : 1.6
 #--------------------------------------------------------------
 sub fetch_seq{
    my @in=@_;
    my $FASTA_index, $FASTA, $where_index, %index, $question, $i,
-	  $s,$t,$fasta,$index_file, $all, $target, $matched, $seq, $gdf, $list, $count, $create;
+	  $s,$t,$fasta,$index_file, $all, $species,$target, $matched, $seq, $gdf, $list, $count, $create;
    my $SEQ_size_max=100000000;
-   if(@_ < 1){ &HELP_fetch_seq;
-	 sub HELP_fetch_seq{
-	   print "\n Usage: $0 [-cafghl] <any swissprot name entry>\n";
-	   print "  -f is for FASTA output format only\n";
-	   print "  -a is for ALL matched seq. \(same as using glob=\> *YEAST\)\n";
-	   print "  -c is for Creating seq.idx file\n";
-	   print "  -h is for HELP!\n";
-	   print "  -g is for GDF file format output\n";
-	   print "  -l is for list of match entries\(in 1 column\)\n";
-	   print "  n= is for Number of seq you want to get from swissprot\n";
-	   print "  s= is for Size limit. Min seq size in swiss, s=10  -> minimum 11 aa seq.\n";
-	   print "  S= is for Size limit. Max seq size in swiss, s=1000 -> get less than 1000 aa seq\n";
-	   print "\n\n $0 utilizes 2 ENV setting if exist: SWDIR or SWISS and SWINDEX";
-	   print "\n   You have to set ENV vars, SWDIR or SWISS to seq.dat path\n";
-	   print "\n   In  csh: \'setenv SWDIR /your/DB/swiss\'";
-	   print "\n   In bash: \'export SWDIR=/your/DB/swiss\'\n";
-	   print "\n   Option: you can also set ENV var, SWINDEX to seq.idx path\n";
-	   print "   You can put absolute path for seq.dat and(or) seq.idx at prompt\n\n\n";
-	   print chr(7);
-	 }
+
+   if(@_ < 1){	  &HELP_fetch_seq;
    }else{
 	 F: for($t=0; $t<@in; $t++){ #'''''''''''' PROMPT ARGV processing ''''''''''''''''''
 		if($in[$t]=~/^\-c$/i){
@@ -94,11 +77,13 @@ sub fetch_seq{
 		   print "  or $0 -c my_db.fa\n\n";
 		   next; }
 		if($in[$t]=~/^\-af$/){ $fasta=$all=1; splice(@in, $t, 1); $t--; next; }
+		if($in[$t]=~/^\-afs$/){ $species=$fasta=$all=1; splice(@in, $t, 1); $t--; next; }
 		if($in[$t]=~/^\-ag$/){ $gdf=$all=1; splice(@in, $t, 1); $t--; next; }
 		if($in[$t]=~/^\-g$/){    $gdf=1; splice(@in, $t, 1); $t--; next; }
 		if($in[$t]=~/^\-f$/i){   $fasta=1; splice(@in, $t, 1); $t--; next; }
 		if($in[$t]=~/^\-a$/i){   $all=1;   splice(@in, $t, 1); $t--; next; }
 		if($in[$t]=~/^\-l$/i){   $list=$all=1;   splice(@in, $t, 1); $t--; next; }
+		if($in[$t]=~/^\-s$/i){  $species=$all=1; splice(@in, $t, 1); $t--; next; }
 		if( ($in[$t]=~/seq\.dat/)&&(-f $in[$t])){ ## if the path for swiss prot is given
 		   $DB=$in[$t];  splice(@in, $t, 1); $t--; next;        }
 		if( ($in[$t]=~/seq\.idx/)&&(-e $in[$t])){ ## if the path for swiss index is given
@@ -177,8 +162,16 @@ sub fetch_seq{
 		 goto ASK;
 	  }elsif(-e "$ENV{'SWDIR'}seq.dat" ){
 		 $DB="$ENV{'SWDIR'}seq.dat";
+	  }elsif(-e "$ENV{'FETCHSWISS'}seq.dat" ){
+		 $DB="$ENV{'FETCHSWISS'}seq.dat";
+	  }elsif(-e "$ENV{'FETCHSWISS'}" ){
+		 $DB="$ENV{'FETCHSWISS'}";
+	  }elsif(-e "$ENV{'SWDIR'}\/seq.dat" ){
+		 $DB="$ENV{'SWDIR'}\/seq.dat";
 	  }elsif( -f "$ENV{'SWISS'}seq.dat" ){
 		 $DB="$ENV{'SWISS'}seq.dat";
+	  }elsif( -f "$ENV{'SWISS'}\/seq.dat" ){
+		 $DB="$ENV{'SWISS'}\/seq.dat";
 	  }elsif( -e 'seq.dat'){
 		 $DB="seq.dat";
 	  }elsif( -f "$ENV{'swiss'}seq.dat"){
@@ -215,10 +208,16 @@ sub fetch_seq{
 		 goto W;
 	  }elsif(defined($FASTA_index)&&(-T $FASTA)){
 		 goto MAIN_SEARCH;
+	  }elsif(-e "$ENV{'FETCHSWISSINDEX'}seq.idx" ){
+		 $index_file="$ENV{'FETCHSWISSINDEX'}seq.idx";
+	  }elsif(-e "$ENV{'FETCHSWISSINDEX'}\/seq.idx" ){
+		 $index_file="$ENV{'FETCHSWISSINDEX'}\/seq.idx";
 	  }elsif(-e "$ENV{'SWDIR'}seq.idx" ){
 		 $index_file="$ENV{'SWDIR'}seq.idx";
 	  }elsif( -f "$ENV{'SWISS'}seq.idx" ){
 		 $index_file="$ENV{'SWISS'}seq.idx";
+	  }elsif( -f "$ENV{'SWISS'}\/seq.idx" ){
+		 $index_file="$ENV{'SWISS'}\/seq.idx";
 	  }elsif( -f "$ENV{'SWINDEX'}" ){
 		 $index_file="$ENV{'SWINDEX'}";
 	  }elsif( -e 'seq.idx'){
@@ -334,58 +333,76 @@ sub fetch_seq{
    #""""""""""""""""""""""""""" MAIN SERACH """""""""""""""""""""""""""""""""""""""""""""""
    MAIN_SEARCH:
    for($i=0; $i<@in; $i++){
-	  my @possible, @pos, %possible; my $target=$in[$i];
+	  my (@possible, @pos, %possible); my $target=$in[$i];
 	  if($target=~/\*/){
 		 $target=~s/\*/\\\w\{0,6\}/; # to handle glob input
 		 $all=1;
 	  }
 	  if(defined($index_file)){
 		 open(INDEX, "$index_file");
-		 while(<INDEX>){
-		   if( /(\w*$target\w*) +(\d+)/ ){ $possible{$1}=$2; }
+		 if($species==1){
+		    while(<INDEX>){
+		      if( /(\w*\_$target) +(\d+)/ ){ $possible{$1}=$2; }
+		    }
+		 }else{
+		    while(<INDEX>){
+		      if( /(\w*$target\w*) +(\d+)/ ){ $possible{$1}=$2; }
+		    }
 		 }
 		 close INDEX;
 		 goto SWISS;
 	  }elsif(($fasta==1) && (defined($FASTA_index)) ){
 		 open(INDEX, "$FASTA_index");
-		 while(<INDEX>){
-		   if( /(\w*$target\w*) +(\d+)/ ){$possible{$1}=$2; }
+		 if($species==1){
+		    while(<INDEX>){
+		      if( /(\w*\_$target) +(\d+)/ ){ $possible{$1}=$2; }
+		    }
+		 }else{
+		    while(<INDEX>){
+		      if( /(\w*$target\w*) +(\d+)/ ){ $possible{$1}=$2; }
+		    }
 		 }
 		 close INDEX;
 		 goto FASTA;
 	  }
 
 	  SWISS:
-	  @poss = keys %possible;
+	  @poss = sort keys %possible;
+
 	  if( (@poss >1)&&($all !=1)){
 		 print "\n @poss","\n";
 		 print chr(7);
 		 print "\n There are more than a few seqs for $in[$i]";
 		 print "\n be more specific! OR use -a option for all matched\n\n";
+		 exit;
 	  }elsif($all !=1){
 		 print "\n";
 		 open (DB, "$DB");
-		 if(defined($SEQ_num_to_fetch)){ $num_sequence=$SEQ_num_to_fetch;
+		 if(defined($SEQ_num_to_fetch)){
+			print "\n# You defined the number of sequence to fetch: $SEQ_num_to_fetch\n";
+			$num_sequence=$SEQ_num_to_fetch;
 		 }else{ $num_sequence=@poss; }
+
 		 A:for($p=0; $p < $num_sequence; $p++){
 		   if($poss[$p]=~/\w*$target\w*/){
-			 $matched=$possible{$poss[$p]};
-			 seek(DB, ($matched-51), 0);
+			 $matched=$possible{$poss[$p]};   # %possible has the name and index num
+			 seek(DB, ($matched-52), 0);
 			 while(<DB>){
 			   if($gdf==1){
 			      if(/ID[\t ]+$poss[$p] +\S+ +\S+ +(\d+)/){
 			         printf ("%-24s %-3d %-7d %-14s %4s\n", "$poss[$p]\/1\-$1", 1, $1, $poss[$p], '0.0');
 					 next A;
 			      }
-			   }elsif((/^\/\/$/)&&($fasta==1)){
+			   }
+			   elsif(/^ {0,2}\/\// and  $fasta !=1){  # !!! DO NOT put $ in /^ {0,2}\/\// as there is something
+				  print "\/\/\n";
+				  next A;
+			   }elsif(/^ {0,2}\/\//  and  $fasta==1){ # !!! DO NOT put $ in /^ {0,2}\/\// as there is something
 				  $seq=~s/ //g;
 				  if( ($SEQ_size_min < length($seq))&&(length($seq) < $SEQ_size_max) ){
 					 print "\>$poss[$p]\n$seq\n"; $seq=''; next A;
 				  }else{  $seq=''; $num_sequence++;  next A; }
-			   }elsif((/^\/\/$/) && ($fasta !=1)){
-				  print "\/\/\n";
-				  next A;
-			   }elsif(($fasta==1)&&(/^[\t ]+\w+/)){
+			   }elsif( $fasta==1 and /^[\t ]+\w+/){
 				  $seq.=$_;
 				  next ;
 			   }elsif($list==1){
@@ -415,12 +432,12 @@ sub fetch_seq{
 			         printf ("%-24s %-3d %-7d %-14s %4s\n", "$poss[$p]\/1\-$1", 1, $1, $poss[$p], '0.0');
 					 next A;
 			      }
-			   }elsif((/^\/\/$/)&&($fasta==1)){
+			   }elsif(/^ {0,2}\/\// and $fasta==1){ # !!! DO NOT put $ in /^ {0,2}\/\// as there is something
 				  $seq=~s/ //g;
 				  if( ($SEQ_size_min < length($seq))&&(length($seq) < $SEQ_size_max) ){
 					 print "\>$poss[$p]\n$seq\n"; $seq='';  next A;
 				  }else{  $seq=''; $num_sequence++; next A; }
-			   }elsif((/^\/\/$/) && ($fasta !=1)){
+			   }elsif(/^ {0,2}\/\// and $fasta !=1){  # !!! DO NOT put $ in /^ {0,2}\/\// as there is something
 				  print "\/\/\n";
 				  next A;
 			   }elsif(($fasta==1)&&(/^[\t ]+\w+/)){
@@ -441,24 +458,25 @@ sub fetch_seq{
 	  }
 
 	  FASTA:
-	  @poss = keys %possible;
+	  @poss = sort keys %possible;
 	  if( (@poss >1)&&($all !=1)){
 		 print "\n @poss","\n";
 		 print chr(7);
 		 print "\n There are more than a few seqs for $in[$i]";
 		 print "\n be more specific! OR use -a option for all matched\n\n";
+		 exit;
 	  }elsif($all !=1){
 		 print "\n";
 		 open (FAS, "$FASTA");
 		 B:for($p=0; $p < @poss; $p++){
-		   if($poss[$p]=~/\w*$target\w*/){
+		 if($poss[$p]=~/\w*$target\w*/){
 			 $matched=$possible{$poss[$p]};
-			 seek(FAS, ($matched-90), 0);
+			 seek(FAS, ($matched-350), 0);
 			 my $seq_found;
 			 while(<FAS>){
-				if((/^> *(\S+)/)&&($seq_found==1)){
+			if((/^> {0,4}(\S+)/)&&($seq_found==1)){
 				   next B;
-	 			}elsif(/^> *(\S+)/){
+	 			}elsif(/^> {0,4}($poss[$p])/){
 				   print;
 				   $seq_found=1;
 				}elsif($seq_found==1){
@@ -474,12 +492,12 @@ sub fetch_seq{
 		 B2:for($p=0; $p < @poss; $p++){
 		   if($poss[$p]=~/\w*$target\w*/){
 			 $matched=$possible{$poss[$p]};
-			 seek(FAS, ($matched-80), 0);
+			 seek(FAS, ($matched-350), 0);
 			 my $seq_found;
 			 while(<FAS>){
-				if((/^> *(\S+)/)&&($seq_found==1)){
+				if((/^> {0,4}(\S+)/)&&($seq_found==1)){
 				   next B2;
-				}elsif(/^> *(\S+)/){
+				}elsif(/^> *($poss[$p])/){
 				   print;
 				   $seq_found=1;
 				}elsif($seq_found==1){
@@ -494,3 +512,98 @@ sub fetch_seq{
 }
 
 
+
+#__________________________________________________________________
+# Title     : HELP_fetch_seq
+# Usage     :
+# Function  :
+# Example   :
+# Warning   : You MUST NOT delete '# options : ..' entry
+#              as it is read  by various subroutines.
+# Class     :
+# Keywords  :
+# Options   : _  for debugging.
+#             #  for debugging.
+# Package   : Bio
+# Reference : http://sonja.acad.cai.cam.ac.uk/perl_for_bio.html
+# Returns   :
+# Tips      :
+# Argument  :
+# Todo      :
+# Author    : A Scientist
+# Version   : 1.0
+# Used in   :
+# Enclosed  :
+#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+sub HELP_fetch_seq{
+	 print "\n Usage: $0 [-cafghl] <any swissprot name entry>\n\n";
+	 print "  -f is for FASTA output format only\n";
+	 print "  -a is for ALL matched seq. \(same as using glob=\> *YEAST\)\n";
+	 print "  -c is for Creating seq.idx file\n";
+	 print "  -h is for HELP!\n";
+	 print "  -g is for GDF file format output\n";
+	 print "  -l is for list of match entries\(in 1 column\)\n";
+	 print "  -s is for species option (input name mst be species (YEAST, RAT, HUMAN..)\n";
+	 print "  n= is for Number of seq you want to get from swissprot\n";
+	 print "  s= is for Size limit. Min seq size in swiss, s=10  -> minimum 11 aa seq.\n";
+	 print "  S= is for Size limit. Max seq size in swiss, s=1000 -> get less than 1000 aa seq\n";
+	 print "\n\n $0 uses ENV vars if exist for seq.dat and seq.idx";
+	 print "\n     FETCHSWISS, SWDIR or SWISS to seq.dat or its path";
+	 print "\n     SWINDEX or FETCHSWISSINDEX to seq.idx or its path\n";
+	 print "\n   Option: set ENV vars, FASTADB, FASTAINDEX to your fasta db and fasta idx files\n";
+	 print "\n   e.g. favourite fasta db and index file: seq.fa, seq.fa.idx in /usr/me/db/seq.fa\n";
+	 print "\n        Your ENV set: setenv FASTADB=/usr/me/db/seq.fa\n";
+	 print "\n        Your ENV set: setenv FASTAINDEX=/usr/me/db/seq.fa.idx\n";
+	 print "\n   fetch also looks at your home dir and pwd for seq.dat, seq.idx \n";
+	 print "\n   In  csh: \'setenv SWISS /your/DB/swiss/\'";
+	 print "\n   In bash: \'export SWISS=/your/DB/swiss/\'\n";
+	 print "   You can put absolute path for seq.dat and(or) seq.idx at prompt\n\n\n";
+	 print chr(7);
+}
+
+
+
+#________________________________________________________________________
+# Title     : read_dir_names_only
+# Usage     : @all_dirs_list = @{&read_dir_names_only(\$absolute_path_dir_name, ....)};
+# Function  : read any dir names and and then put in array.
+# Example   :
+# Warning   : This does not report '.', '..'
+#             Only file names are reported. Compare with &read_any_dir
+# Class     :
+# Keywords  :
+# Options   :
+# Package   :
+# Reference :
+# Returns   : one ref. of array.
+# Tips      :
+# Argument  : takes one or more scaler references. ('.', \$path, $path, ... )
+# Todo      :
+# Author    : A Biomatic
+# Version   : 3.1
+# Used in   :
+# Enclosed  :
+#--------------------------------------------------------------------
+sub read_dir_names_only{
+  my($in_dir, $i,$k, @possible_dirs,
+	  @final_files, $full_dir, $pwd, $path,@read_files);
+  $pwd=`pwd`; chomp($pwd); $full_dir=1;
+  for($k=0; $k < @_; $k++){
+	 if   ( ($_[$k] eq '.') || !(defined($_[$k]))){  $in_dir=$pwd;  }
+	 elsif(!(ref($_[$k]))){   $in_dir=$_[$k];   }
+	 elsif(ref($_[$k])){      $in_dir =${$_[$k]};    }
+	 if($in_dir =~ /^([\w\-\.]+)$/){  $in_dir="$pwd\/$in_dir"; $full_dir = 0; }
+	 else{ $full_dir =1; }
+	 ##########  Main READING PART ##########
+	 opendir(DIR1,"$in_dir");
+	 @read_files = readdir(DIR1);
+	 for($i=0; $i < @read_files; $i ++){
+		$read_files[$i]="$in_dir\/$read_files[$i]";
+		if( ($read_files[$i] !~ /\/\.\.?$/) && ( -d $read_files[$i]) ){
+		  $read_files[$i]=~s/\.\///; ## removing ./ in front of dirs (in bash)
+		  push(@final_files, "$read_files[$i]");
+		}
+	 }
+  }
+  return([sort @final_files]);
+}
